﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;


namespace MyDNSUpdate
{
    class DDNSClient
    {
        public enum DDNSClientResult
        {
            NOACTION,
            UPDATED,
            FAILED,
        }

        AccountSettings setting = new AccountSettings();
        TimeSpan retryInterval;
        string lastResultLog;
        DateTime innerSchedule;
        public bool Enabled { get { return setting.Enabled; } }

        public DDNSClient(AccountSettings setting_I, TimeSpan retryInterval_I)
        {
            setting = setting_I;
            retryInterval = retryInterval_I;
            innerSchedule = DateTime.MinValue;
        }

        public DDNSClientResult Check(IPAddress addr, bool force)
        {
            DDNSClientResult result = DDNSClientResult.NOACTION;

            if (force || (innerSchedule < DateTime.Now)) {

                result = DDNSClientResult.FAILED;

                HttpWebResponse webres = null;
                Stream st = null;
                StreamReader sr = null;

                try {
                    String URL = setting.detail.URL;
                    for (int c = 0; c < setting.detail.HTTPParams.Count; c++) {
                        URL += (c == 0) ? "?" : "&";
                        URL += setting.detail.HTTPParams[c].Name + "=" + setting.detail.HTTPParams[c].Value;
                    }
                    HttpWebRequest webreq = (HttpWebRequest)WebRequest.Create(URL);
                    if (setting.detail.BasicAuth) {
                        webreq.Credentials = new System.Net.NetworkCredential(setting.detail.Username, setting.detail.Password);
                        webreq.PreAuthenticate = true;
                    }
                    webreq.Proxy = WebRequest.GetSystemWebProxy();
                    webres = (System.Net.HttpWebResponse)webreq.GetResponse();
                    st = webres.GetResponseStream();
                    sr = new StreamReader(st);

                    string resContent = sr.ReadToEnd();
                    if ((resContent.Length > 0) && (resContent.IndexOf(setting.detail.OK) >= 0)) {
                        lastResultLog = "Succeeded.";
                        result = DDNSClientResult.UPDATED;
                    }
                    else {
                        lastResultLog = "Bad response." + resContent;
                    }
                }
                catch (UriFormatException) {
                    lastResultLog = "Invalid URL.";
                }
                catch (WebException) {
                    lastResultLog = "Timeout.";
                }
                catch (ProtocolViolationException) {
                    lastResultLog = "No response stream.";
                }
                catch (IOException) {
                    lastResultLog = "I/O Error.";
                }
                catch (ArgumentNullException) {
                    lastResultLog = "No content.";
                }
                catch (Exception e) {
                    lastResultLog = "Failed : " + e.Message;
                }
                finally {
                    if (sr != null) {
                        sr.Close();
                        sr.Dispose();
                    }
                    if (st != null) {
                        st.Close();
                        st.Dispose();
                    }
                    if (webres != null) webres.Close();
                }
                setting.LastUpdate = DateTime.Now;
                setting.LastStatus = (result == DDNSClientResult.UPDATED);

                innerSchedule += NextDelay();
                if (force || (innerSchedule < DateTime.Now)) innerSchedule = DateTime.Now + NextDelay();
            }

            return result;
        }

        private TimeSpan NextDelay()
        {
            return (setting.LastStatus ? (TimeSpan)setting.Interval : retryInterval) - TimeSpan.FromMinutes(1);
        }

        public void GetLastLog(out string log)
        {
            log = lastResultLog;
        }
    }
}
