﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Reflection;
using System.Diagnostics;

namespace MyDNSUpdate
{
    public partial class Form1 : Form
    {
        bool running = false;
        Settings settings = new Settings();
        IPChecker ipChecker;
        DDNSController controller;
        private Unclassified.UI.SystemMenu systemMenu;

        bool autoStart = false;

        delegate void StatusStripDelegate(string value, SystemIconEx.SystemIcons icon);
        delegate void DataGridViewUpdateDelegate();

        public Form1(string arg)
        {
            if (arg == "a") autoStart = true;

            InitializeComponent();

            systemMenu = new Unclassified.UI.SystemMenu(this);
            systemMenu.AddCommand("&About " + Application.ProductName + "...", OnSysMenuAbout, true);
        }

        private void dataGridView1_DoubleClick(object sender, EventArgs e)
        {
            if (dataGridView1.SelectedRows.Count == 0) return;
            if (running) return;

            foreach (DataGridViewRow r in dataGridView1.SelectedRows) {
                AccountSettings tmpAccount;
                bool add = false;
                if (r.Index < settings.Accounts.Count) {
                    tmpAccount = settings.Accounts[r.Index];
                }
                else {
                    tmpAccount = new AccountSettings();
                    add = true;
                }
                Form2 form = new Form2(tmpAccount);
                if (form.ShowDialog() == DialogResult.OK) {
                    if (add) settings.Accounts.Add(form.account);
                    else settings.Accounts[r.Index] = form.account;
                }
            }
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            Settings.SaveSettings(settings);
        }

        private void buttonAdd_Click(object sender, EventArgs e)
        {
            AccountSettings tmpAccount = new AccountSettings();
            Form2 form = new Form2(tmpAccount);
            if (form.ShowDialog() == DialogResult.OK) {
                settings.Accounts.Add(form.account);
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            string log;
            if (!Settings.LoadSettings(ref settings, out log)) {
                MessageBox.Show(log, Program.appName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.Close();
            }
            else {
                this.Icon = Icon.ExtractAssociatedIcon(Assembly.GetExecutingAssembly().Location);
                this.MinimumSize = this.Size;
                this.Text = Application.ProductName;

                ipChecker = new IPChecker();

                dataGridView1.DataSource = settings.Accounts;
                dataGridView1.Columns[2].ReadOnly = true;
                dataGridView1.Columns[3].ReadOnly = true;
                dataGridView1.Columns[4].Visible = false;
                dataGridView1.Columns[5].Visible = false;
                dataGridView1.AutoResizeColumn(0, DataGridViewAutoSizeColumnMode.AllCells);
                dataGridView1.AutoResizeColumn(1, DataGridViewAutoSizeColumnMode.AllCells);
                dataGridView1.AutoResizeColumn(2, DataGridViewAutoSizeColumnMode.AllCells);

                textBoxIPURL.Text = settings.IPURL;
                timeSpanPickerIPInterval.MinimumValue = TimeSpan.FromMinutes(5);
                timeSpanPickerIPInterval.ValueTimeSpan = settings.IPInterval;
                timeSpanPickerIPInterval.ValueChanged += new EventHandler(Form1_SettingChanged);
                timeSpanPickerRetryInterval.MinimumValue = TimeSpan.FromMinutes(5);
                timeSpanPickerRetryInterval.ValueTimeSpan = settings.RetryInterval;
                timeSpanPickerRetryInterval.ValueChanged += new EventHandler(Form1_SettingChanged);

                switch(IsAdmin.GetAdministrativeCapability())
                {
                case IsAdmin.ADMINISTRATIVE_CAPABILITY.REGULAR_USER:
                    buttonRegister.Enabled = false;
                    break;
                case IsAdmin.ADMINISTRATIVE_CAPABILITY.ELEVATION_REQUIRED:
                    SystemIconEx sysIconEx = new SystemIconEx(SystemIconEx.SystemIcons.Shield, SystemInformation.SmallIconSize);
                    buttonRegister.TextImageRelation = TextImageRelation.ImageBeforeText;
                    buttonRegister.Image = sysIconEx.Icon.ToBitmap();
                    sysIconEx.Dispose();
                    break;
                }

                if (autoStart) ToggleIPChecker();

                UpdateUIState();
            }
        }

        private void Form1_SettingChanged(object sender, EventArgs e)
        {
            if(sender == textBoxIPURL ) {
                settings.IPURL = textBoxIPURL.Text;
            }
            else if (sender == timeSpanPickerIPInterval) {
                settings.IPInterval = timeSpanPickerIPInterval.ValueTimeSpan;
            }
            else if (sender == timeSpanPickerRetryInterval) {
                settings.RetryInterval = timeSpanPickerRetryInterval.ValueTimeSpan;
            }
        }

        private void buttonDelete_Click(object sender, EventArgs e)
        {
            if (dataGridView1.SelectedRows.Count == 0) return;

            if (CenterMessageBox.Show(this, "Are you sure you want to delete the account?", Program.appName, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes) {
                foreach (DataGridViewRow r in dataGridView1.SelectedRows) {
                    settings.Accounts.RemoveAt(r.Index);
                }
            }
        }

        private void UpdateUIState()
        {
            textBoxIPURL.Enabled = !running;
            timeSpanPickerIPInterval.Enabled = !running;
            timeSpanPickerRetryInterval.Enabled = !running;
            //dataGridView1.Enabled = !running;
            dataGridView1.ReadOnly = running;
            buttonAdd.Enabled = !running;
            buttonDelete.Enabled = !running;
            buttonRegister.Enabled = !running;
            buttonStart.Text = running ? "Stop" : "Start";
            buttonRegister.Text = Service1.IsRegistered() ? "Unregister the service" : "Register as a service";
        }

        private void buttonStart_Click(object sender, EventArgs e)
        {
            ToggleIPChecker();
            UpdateUIState();
        }

        private void ToggleIPChecker()
        {
            running = !running;
            if (running) {
                controller = new DDNSController(settings);
                ipChecker.checkerURL = settings.IPURL;
                ipChecker.interval = settings.IPInterval;
                ipChecker.resultEvent += Form1_IPResult;
                ipChecker.Start();
            }
            else {
                ipChecker.Stop();
                controller.Clear();
            }
        }

        public void SetToolStripStatusLabelIP(string value, SystemIconEx.SystemIcons icon)
        {
            toolStripStatusLabelIP.Text = value;
            if (icon != SystemIconEx.SystemIcons.None) {
                SystemIconEx sysIconEx = new SystemIconEx(icon, SystemInformation.SmallIconSize);
                toolStripStatusLabelIP.Image = sysIconEx.Icon.ToBitmap();
                sysIconEx.Dispose();
            }
            else {
                toolStripStatusLabelIP.Image = null;
            }
        }
        public void SetToolStripStatusLabelLog(string value, SystemIconEx.SystemIcons icon)
        {
            toolStripStatusLabelLog.Text = value + "(" + DateTime.Now.ToString() +")";
            if (icon != SystemIconEx.SystemIcons.None) {
                SystemIconEx sysIconEx = new SystemIconEx(icon, SystemInformation.SmallIconSize);
                toolStripStatusLabelLog.Image = sysIconEx.Icon.ToBitmap();
                sysIconEx.Dispose();
            }
            else {
                toolStripStatusLabelLog.Image = null;
            }
        }
        public void DataGridViewUpdate()
        {
            dataGridView1.Invalidate();
        }

        public void Form1_IPResult(IPCheckerResult result, IPAddress addr, string log)
        {
            switch (result) {
            case IPCheckerResult.SUCCESS_CHENGED:
                BeginInvoke(new StatusStripDelegate(SetToolStripStatusLabelIP), addr.ToString(), SystemIconEx.SystemIcons.Information);
                BeginInvoke(new StatusStripDelegate(SetToolStripStatusLabelLog), log, SystemIconEx.SystemIcons.None);
                controller.Check(addr, true);
                BeginInvoke(new DataGridViewUpdateDelegate(DataGridViewUpdate));
                break;
            case IPCheckerResult.SUCCESS_SAME:
                BeginInvoke(new StatusStripDelegate(SetToolStripStatusLabelIP), addr.ToString(), SystemIconEx.SystemIcons.None);
                BeginInvoke(new StatusStripDelegate(SetToolStripStatusLabelLog), log, SystemIconEx.SystemIcons.None);
                controller.Check(addr, false);
                BeginInvoke(new DataGridViewUpdateDelegate(DataGridViewUpdate));
                break;
            case IPCheckerResult.FAILED:
                BeginInvoke(new StatusStripDelegate(SetToolStripStatusLabelLog), log, SystemIconEx.SystemIcons.Error);
                break;
            }
        }

        private void dataGridView1_CellErrorTextNeeded(object sender, DataGridViewCellErrorTextNeededEventArgs e)
        {
            DataGridView dgv = (DataGridView)sender;
            //if ((e.ColumnIndex == 3) || (e.ColumnIndex == 4)) {
            if (e.ColumnIndex == 3) {
                if ((bool)dgv[4, e.RowIndex].Value == false) {
                    //dgv[3, e.RowIndex].ErrorText = "Failed.";
                    if (controller != null) {
                        string log = "";
                        controller.GetLastLog(e.RowIndex, ref log);
                        dgv[3, e.RowIndex].ErrorText = log;
                    }
                    else {
                        dgv[3, e.RowIndex].ErrorText = "";
                    }
                }
                else {
                    dgv[3, e.RowIndex].ErrorText = "";
                }
            }
        }

        private void buttonRegister_Click(object sender, EventArgs e)
        {
            bool exist = Service1.IsRegistered();
            bool status = false;

            switch (IsAdmin.GetAdministrativeCapability()) {
            case IsAdmin.ADMINISTRATIVE_CAPABILITY.ELEVATION_REQUIRED:
                ProcessStartInfo psi = new ProcessStartInfo();
                psi.FileName = Application.ExecutablePath;
                psi.Arguments = exist ? "u" : "i";
                psi.Verb = "runas";
                try {
                    Process p = Process.Start(psi);
                    p.WaitForExit();
                    status = p.ExitCode == 0 ? true : false;
                }
                catch {
                    MessageBox.Show(exist ? "Unregistration" : "Registration" + " failed");
                }
                break;

            case IsAdmin.ADMINISTRATIVE_CAPABILITY.ADMINISTRATOR:
                if (exist) status = Service1.Unregister();
                else status = Service1.Register();
                break;

            default:
                MessageBox.Show("Administrator privilege is required.");
                break;
            }

            if (status) MessageBox.Show("Succeeded.");
            UpdateUIState();
        }

        protected override void WndProc(ref Message msg)
        {
            base.WndProc(ref msg);

            // Let it know all messages so it can handle WM_SYSCOMMAND
            // (This method is inlined)
            systemMenu.HandleMessage(ref msg);
        }

        private void OnSysMenuAbout()
        {
            AboutForm af = new AboutForm();
            af.ShowDialog();
        }

        private void buttonEdit_Click(object sender, EventArgs e)
        {
            dataGridView1_DoubleClick(null, null);
        }
    }
}
