﻿using System;
using System.Net;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;

namespace MyDNSUpdate
{
    public enum IPCheckerResult
    {
        FAILED,
        SUCCESS_CHENGED,
        SUCCESS_SAME,
    }
    delegate void IPCheckerDelegate(IPCheckerResult result, IPAddress addr, string logMessage);

    class IPChecker
    {
        public event IPCheckerDelegate resultEvent = delegate(IPCheckerResult result, IPAddress addr, string logMessage) { };

        public String checkerURL { get; set; }
        public IPAddress ipaddr { get; private set; }
        public TimeSpan interval { get; set; }
        protected readonly IPAddress ipzero = IPAddress.Parse("0.0.0.0");

        TimerHelper.TimerEventDelegate timerDelegate;
        TimerHelper timer = null;

        public IPChecker()
        {
            ipaddr = IPAddress.None;
            interval = TimeSpan.FromMinutes(10);
        }

        public void Start()
        {
            timerDelegate = new TimerHelper.TimerEventDelegate(timer_elapsed);
            timer = new TimerHelper();
            timer.TimerEvent += timerDelegate;
            timer.Start(interval, true, null);
        }

        public void Stop()
        {
            if ( timer != null ) timer.Stop();
        }

        public void timer_elapsed(System.Threading.Timer timer, object o)
        {
            string log;
            IPCheckerResult result = Check(out log);
            resultEvent(result, ipaddr, log);
        }

        public IPCheckerResult Check(out String logMessage)
        {
            IPCheckerResult retVal = IPCheckerResult.FAILED;
            logMessage = "";
            HttpWebResponse webres = null;
            Stream st = null;
            StreamReader sr = null;

            try {
                HttpWebRequest webreq = (HttpWebRequest)WebRequest.Create(checkerURL); //例外発生あり 特にUriFormatException
                webreq.Proxy = WebRequest.GetSystemWebProxy();
                webres = (HttpWebResponse)webreq.GetResponse(); //色々と例外発生あり
                st = webres.GetResponseStream(); //建前上は例外発生あり
                sr = new StreamReader(st); //建前上は例外発生あり
                IPAddress newIpaddr;

                Regex ipReg = new Regex(@"(([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])", RegexOptions.Multiline);
                Match m = ipReg.Match(sr.ReadToEnd()); //どちらかというとReadToEnd()で例外発生あり
                newIpaddr = IPAddress.Parse(m.Value); //正規表現で絞ってるからたぶん大丈夫

                if (newIpaddr.Equals(ipzero)) {
                    retVal = IPCheckerResult.FAILED;
                    logMessage = "Disconnected.";
                }
                else if (!newIpaddr.Equals(ipaddr)) {
                    retVal = IPCheckerResult.SUCCESS_CHENGED;
                    ipaddr = newIpaddr;
                    logMessage = "Succeess.";
                }
                else {
                    retVal = IPCheckerResult.SUCCESS_SAME;
                    logMessage = "Succeess.";
                }

            }
            catch (UriFormatException) {
                logMessage = "Invalid URL.";
            }
            catch (WebException) {
                logMessage = "Timeout.";
            }
            catch (ProtocolViolationException) {
                logMessage = "No response stream.";
            }
            catch (IOException) {
                logMessage = "I/O Error.";
            }
            catch (ArgumentNullException) {
                logMessage = "No content.";
            }
            catch (FormatException) {
                logMessage = "Format invalid.";
            }
            catch (Exception e) {
                logMessage = "Failed : " + e.Message;
            }
            finally {
                if (sr != null) {
                    sr.Close();
                    sr.Dispose();
                }
                if (st != null) {
                    st.Close();
                    st.Dispose();
                }
                if (webres != null) webres.Close();
            }

            return retVal;
        }
    }
}
