﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.ServiceProcess;
using System.Text;
using System.Reflection;
using System.Net;
using System.Configuration.Install;
using System.Windows.Forms;

namespace MyDNSUpdate
{
    public partial class Service1 : ServiceBase
    {
        Settings settings = new Settings();
        IPChecker ipChecker = new IPChecker();
        DDNSController controller;

        public Service1()
        {
            InitializeComponent();
        }

        protected override void OnStart(string[] args)
        {
            string log;

            if (!Settings.LoadSettings(ref settings, out log)) {
                EventLog.WriteEntry(Program.appName, log, EventLogEntryType.Error);
                controller = new DDNSController(settings);
                this.Stop();
            }
            else if (settings.Accounts.Count == 0) {
                EventLog.WriteEntry(Program.appName, "No DDNS Account.", EventLogEntryType.Error);
                controller = new DDNSController(settings);
                this.Stop();
            }
            else {
                controller = new DDNSController(settings);
                ipChecker.checkerURL = settings.IPURL;
                ipChecker.interval = settings.IPInterval;
                ipChecker.resultEvent += IPResult;
                ipChecker.Start();
            }
        }

        public void IPResult(IPCheckerResult result, IPAddress addr, string log)
        {
            DDNSClient.DDNSClientResult[] ddnsResult;
            string msg = "";

            switch (result) {
            case IPCheckerResult.SUCCESS_CHENGED:
                EventLog.WriteEntry(Program.appName, "IP Address changed : " + addr.ToString(), EventLogEntryType.Information);
                ddnsResult = controller.Check(addr, true);
                for (int c = 0; c < ddnsResult.Length; c++) {
                    if (ddnsResult[c] != DDNSClient.DDNSClientResult.NOACTION) {
                        controller.GetLastLog(c, ref msg);
                        EventLog.WriteEntry(Program.appName, settings.Accounts[c].Title + " : " + msg, ddnsResult[c] == DDNSClient.DDNSClientResult.UPDATED ? EventLogEntryType.Information : EventLogEntryType.Error);
                    }
                }
                break;
            case IPCheckerResult.SUCCESS_SAME:
                ddnsResult = controller.Check(addr, false);
                for (int c = 0; c < ddnsResult.Length; c++) {
                    if (ddnsResult[c] != DDNSClient.DDNSClientResult.NOACTION) {
                        controller.GetLastLog(c, ref msg);
                        EventLog.WriteEntry(Program.appName, settings.Accounts[c].Title + " : " + msg, ddnsResult[c] == DDNSClient.DDNSClientResult.UPDATED ? EventLogEntryType.Information : EventLogEntryType.Error);
                    }
                }
                break;
            case IPCheckerResult.FAILED:
                EventLog.WriteEntry(Program.appName, "IP Check : " + log, EventLogEntryType.Error);
                break;
            }
        }

        protected override void OnStop()
        {
            ipChecker.Stop();
            controller.Clear();
        }

        public static bool IsRegistered()
        {
            bool exist = false;
            ServiceBase dummyService = new Service1();
            String name = dummyService.ServiceName;
            dummyService.Dispose();
            ServiceController[] services = ServiceController.GetServices();
            foreach (ServiceController sc in services) {
                if (sc.ServiceName == name) {
                    exist = true;
                    break;
                }
            }
            return exist;
        }

        public static bool Register()
        {
            bool status = false;
            try {
                ManagedInstallerClass.InstallHelper(new string[] { Assembly.GetExecutingAssembly().Location });
                status = true;
            }
            catch (InvalidOperationException) {
                MessageBox.Show("Registration requires admin privilege.", Program.appName, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception e) {
                MessageBox.Show(e.Message);
            }
            return status;
        }

        public static bool Unregister()
        {
            bool status = false;
            try {
                ManagedInstallerClass.InstallHelper(new string[] { "/u", Assembly.GetExecutingAssembly().Location });
                status = true;
            }
            catch (InvalidOperationException) {
                MessageBox.Show("Unregistration requires admin privilege.", Program.appName, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception e) {
                MessageBox.Show(e.Message);
            }
            return status;
        }
    }
}
