﻿// from http://stackoverflow.com/questions/7483233/net-how-to-serialize-a-timespan-to-xml-not-the-same-question

using System;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;

[Serializable, XmlSchemaProvider("TimeSpanSchema")]
public class XmlTimeSpan : IXmlSerializable
{
    private TimeSpan _value;

    public XmlTimeSpan()
    {
        _value = TimeSpan.Zero;
    }

    public XmlTimeSpan(TimeSpan value)
    {
        _value = value;
    }

    public XmlSchema GetSchema()
    {
        return null;
    }

    public void ReadXml(XmlReader reader)
    {
        _value = XmlConvert.ToTimeSpan(reader.ReadElementContentAsString());
    }

    public void WriteXml(XmlWriter writer)
    {
        writer.WriteValue(XmlConvert.ToString(_value));
    }

    public static XmlQualifiedName TimeSpanSchema(XmlSchemaSet xs)
    {
        return new XmlQualifiedName("duration", "http://www.w3.org/2001/XMLSchema");
    }

    public static implicit operator TimeSpan?(XmlTimeSpan o)
    {
        return o == null ? default(TimeSpan?) : o._value;
    }

    public static implicit operator XmlTimeSpan(TimeSpan? o)
    {
        return o == null ? null : new XmlTimeSpan { _value = o.Value };
    }

    public static implicit operator TimeSpan(XmlTimeSpan o)
    {
        return o == null ? default(TimeSpan) : o._value;
    }

    public static implicit operator XmlTimeSpan(TimeSpan o)
    {
        return o == default(TimeSpan) ? null : new XmlTimeSpan { _value = o };
    }

    public override string ToString()
    {
        return _value.ToString();
    }

    public override bool Equals(object obj)
    {
        if (obj.GetType() != this.GetType()) return false;
        return _value.Equals(((XmlTimeSpan)obj)._value);
    }

    public override int GetHashCode()
    {
        return _value.GetHashCode();
    }
}
